import {
  Component,
  OnInit,
  Inject,
  ViewChild,
  ElementRef
} from '@angular/core';
import { MatDialogRef, MAT_DIALOG_DATA } from '@angular/material/dialog';
import { FormGroup, FormBuilder } from '@angular/forms';
import { NGXLogger } from 'ngx-logger';
import { Store, select } from '@ngrx/store';
import * as AppStore from '@app/store';
import {
  Observable,
  combineLatest,
  Subscription,
  of,
  BehaviorSubject
} from 'rxjs';
import {
  DialogService,
  ConfirmDialogComponent,
  ConfirmDialogData,
  ConfirmDialogResult
} from '@ucap-webmessenger/ui';
import { map, tap, catchError, take } from 'rxjs/operators';
import * as SyncStore from '@app/store/messenger/sync';
import {
  PhoneBookSyncType,
  PhoneBookSndNotification,
  PhoneBookInfo,
  KEY_PHONE_BOOK
} from '@ucap-webmessenger/protocol-sync';
import { environment } from '../../../../../environments/environment';
import { SessionStorageService } from '@ucap-webmessenger/web-storage';
import { TranslateService } from '@ngx-translate/core';
import { LocalStorageService } from '@ucap-webmessenger/web-storage';
import { LoginResponse } from '@ucap-webmessenger/protocol-authentication';
import { PiService, PromptMessageStatusCode } from '@ucap-webmessenger/pi';
import { EnvironmentsInfo, KEY_ENVIRONMENTS_INFO } from '@app/types';

export interface PhoneBookDialogData {}

// tslint:disable-next-line: no-empty-interface
export interface PhoneBookDialogResult {
  choice: boolean;
}

@Component({
  selector: 'app-layout-messenger-phone-book',
  templateUrl: './phone-book.dialog.component.html',
  styleUrls: ['./phone-book.dialog.component.scss']
})
export class PhoneBookDialogComponent implements OnInit {
  @ViewChild('messageContainer', { static: true })
  messageContainer: ElementRef;

  loginRes: LoginResponse;
  environmentsInfo: EnvironmentsInfo;

  fgSearch: FormGroup;
  tempAgeLimits = [];

  phoneBookListSubscription: Subscription;

  constructor(
    public dialogRef: MatDialogRef<
      PhoneBookDialogComponent,
      PhoneBookDialogResult
    >,
    @Inject(MAT_DIALOG_DATA) public data: PhoneBookDialogData,
    private store: Store<any>,
    private logger: NGXLogger,
    private formBuilder: FormBuilder,
    private localStorageService: LocalStorageService,
    private sessionStorageService: SessionStorageService,
    private dialogService: DialogService,
    private translateService: TranslateService,
    private piService: PiService
  ) {
    this.phoneBookListSubscription = this.localStorageService
      .observe(KEY_PHONE_BOOK)
      .pipe(
        map((v, i) => {
          this.logger.debug(v);
        })
      )
      .subscribe();

    this.environmentsInfo = this.sessionStorageService.get<EnvironmentsInfo>(
      KEY_ENVIRONMENTS_INFO
    );
  }

  phoneBookNoti: PhoneBookSndNotification;

  phoneBookList: PhoneBookInfo[];
  isPhoneBookEmpty: boolean;

  ngOnInit(): void {
    this.fgSearch = this.formBuilder.group({
      searchInput: null
    });

    this.phoneBookNoti = this.localStorageService.encGet<
      PhoneBookSndNotification
    >(KEY_PHONE_BOOK, environment.customConfig.appKey);

    if (!!this.phoneBookNoti && this.phoneBookNoti.phoneBookInfos.length > 0) {
      this.isPhoneBookEmpty = true;
      this.phoneBookList = this.phoneBookNoti.phoneBookInfos;
    } else {
      this.isPhoneBookEmpty = false;
    }
  }

  async onClickPhoneBookMenu(menuType: string) {
    switch (menuType) {
      case 'SYNC':
        {
          const result = await this.dialogService.open<
            ConfirmDialogComponent,
            ConfirmDialogData,
            ConfirmDialogResult
          >(ConfirmDialogComponent, {
            width: '400px',
            data: {
              title: this.translateService.instant('call.phoneBookSync'),
              html: this.translateService.instant('call.confirmPhoneBookSync')
            }
          });

          if (!!result && !!result.choice && result.choice) {
            this.localStorageService.remove(KEY_PHONE_BOOK);

            this.store.dispatch(
              SyncStore.phoneBookReady({
                req: {
                  destination: PhoneBookSyncType.TO_MOBILE
                }
              })
            );
          }
        }
        break;
      case 'INIT':
        {
          this.localStorageService.remove(KEY_PHONE_BOOK);
        }
        break;
      default:
        break;
    }
  }

  async onClickToCall(callNumber: string) {
    this.logger.debug(callNumber);

    const phoneNum = callNumber.replace(/\D/g, '');

    const result = await this.dialogService.open<
      ConfirmDialogComponent,
      ConfirmDialogData,
      ConfirmDialogResult
    >(ConfirmDialogComponent, {
      data: {
        title: this.translateService.instant('common.messages.confirm'),
        html: this.translateService.instant('call.callWithNumber', {
          phoneNumber: phoneNum
        })
      }
    });

    if (!!result && !!result.choice && result.choice) {
      this.logger.debug(phoneNum);

      this.piService
        .sendCall({
          userSeq: this.loginRes.userSeq,
          deviceType: this.environmentsInfo.deviceType,
          tokenKey: this.loginRes.tokenString,
          calleeNumber: phoneNum
        })
        .pipe(
          take(1),
          map(res => {
            if (res.responseCode === PromptMessageStatusCode.Success) {
              this.logger.debug('SUCCESS');
              this.logger.debug(res);
            } else {
              this.logger.error(res);
            }
          }),
          catchError(error => of(this.logger.debug(error)))
        )
        .subscribe();
    }
  }

  onClickChoice(): void {
    this.dialogRef.close();
  }

  onKeyDownEnter(event: KeyboardEvent, search: string) {
    event.preventDefault();
    event.stopPropagation();

    // if (search.trim().length >= 1) {
    //   this.isSearch = true;
    //   this.searchRoomList = this.roomList.filter(room => {
    //     if (room.roomName.indexOf(search) >= 0) {
    //       return true;
    //     } else if (
    //       this.getRoomUserList(room).filter(user =>
    //         user.seq !== this.loginRes.userSeq
    //           ? user.name.indexOf(search) >= 0
    //           : false
    //       ).length > 0
    //     ) {
    //       return true;
    //     } else {
    //       return false;
    //     }
    //   });
    // } else {
    //   this.isSearch = false;
    //   this.searchRoomList = [];
    // }
    // setTimeout(() => (this.filteredRecommendedWordList = []), 200);
  }
  onClickSearchCancel() {
    // this.isSearch = false;
    // this.searchRoomList = [];
    // this.filteredRecommendedWordList = [];
  }
}
